// src/main/javascript/mock/zinmocks.js
import { JsonBeachhead, ListTraverser, CollectingState } from "/js/ziwsh.js";
var ZinTest = function(runner, bridge, connName) {
  this.logger = runner.logger;
  this.logger.log("creating ZinTest with ", runner);
  this.bridge = bridge;
  this.conn = connName;
  this.servers = {};
};
ZinTest.prototype.data = function(cx, zn, fl) {
  var self = this;
  zn = cx.full(zn);
  fl = cx.full(fl);
  this.bridge.lock("zin.data");
  this.bridge.send({ action: "data", conn: this.conn, requestId: this.bridge.nextRequestId(() => self.bridge.unlock("zin.data")), args: [zn, fl] });
};
ZinTest.prototype.user = function(cx, zn, u) {
  var self = this;
  zn = cx.full(zn);
  u = cx.full(u);
  this.bridge.lock("zin.user");
  this.bridge.send({ action: "user", conn: this.conn, requestId: this.bridge.nextRequestId(() => self.bridge.unlock("zin.user")), args: [zn, u] });
};
ZinTest.prototype.dump = function(cx, zn) {
  zn = cx.full(zn);
  this.jbm.dump(cx, zn);
};
ZinTest.prototype.attach = function(cx, zn, u) {
  zn = cx.full(zn);
  u = cx.full(u);
  var self = this;
  self.ensureServer(zn, u);
};
ZinTest.prototype.hasBench = function(cx, zn, aid, bench) {
  var self = this;
  zn = cx.full(zn);
  aid = cx.full(aid);
  bench = cx.full(bench);
  this.bridge.lock("zin.hasBench");
  this.bridge.send({ action: "hasBench", conn: this.conn, requestId: this.bridge.nextRequestId(() => self.bridge.unlock("zin.hasBench")), args: [zn, aid, bench] });
};
ZinTest.prototype.hasEntityLink = function(cx, zn, bid, eid) {
  var self = this;
  zn = cx.full(zn);
  bid = cx.full(bid);
  eid = cx.full(eid);
  this.bridge.lock("zin.hasEntityLink");
  this.bridge.send({ action: "hasEntityLink", conn: this.conn, requestId: this.bridge.nextRequestId(() => self.bridge.unlock("zin.hasEntityLink")), args: [zn, bid, eid] });
};
ZinTest.prototype.obtainDataEntry = function(cx, zn, ziid, ty, iv) {
  zn = cx.full(zn);
  ziid = cx.full(ziid);
  ty = cx.full(ty);
  var self = this;
  this.bridge.lock("zin.obtainDataEntry");
  var onResponse = (msg) => {
    var s = msg.value;
    self.logger.log("have obtained ", s);
    var jp = JSON.parse(s);
    self.logger.log("parsed as ", jp);
    var trav = new ListTraverser(cx, new CollectingState(cx));
    var jb = new JsonBeachhead(cx, null, null, null);
    try {
      jb.handleArg(trav, cx, jp);
      self.logger.log("concluded with", trav.ret[0]);
      iv.bindActual(trav.ret[0]);
    } catch (e) {
      self.logger.log("error was", e);
      self.logger.log("error at", e.stack);
    }
    self.bridge.unlock("zin.obtainDataEntry");
  };
  this.bridge.send({ action: "obtainDataEntry", conn: this.conn, requestId: this.bridge.nextRequestId(onResponse), args: [zn, ziid, ty, { result: onResponse }] });
};
ZinTest.prototype.obtainPersona = function(cx, zn, arena, bench, identity, ty, iv) {
  zn = cx.full(zn);
  arena = cx.full(arena);
  bench = cx.full(bench);
  identity = cx.full(identity);
  ty = cx.full(ty);
  var self = this;
  this.bridge.lock("zin.obtainPersona");
  var ih = {
    result: function(s) {
      self.logger.log("have obtained persona json ", s);
      var jp = JSON.parse(s);
      self.logger.log("parsed as ", jp);
      var trav = new ListTraverser(cx, new CollectingState(cx));
      var jb = new JsonBeachhead(cx, null, null, null);
      try {
        jb.handleArg(trav, jp);
        self.logger.log("concluded with", trav.ret[0]);
        iv.bindActual(trav.ret[0]);
      } catch (e) {
        self.logger.log("error was", e);
        self.logger.log("error at", e.stack);
      }
    }
  };
  var onResponse = (msg) => {
    ih.result(msg.value);
    self.bridge.unlock("zin.obtainPersona");
  };
  this.bridge.send({ action: "obtainPersona", conn: this.conn, requestId: this.bridge.nextRequestId(onResponse), args: [zn, arena, bench, identity, ty, ih] });
};
ZinTest.prototype.connStatus = function(stat) {
  console.log("connection status", stat);
};
ZinTest.prototype.recvMessage = function(msg) {
  console.log("connection received message", msg);
  try {
    var js = JSON.parse(msg);
    if (js && js.action == "idem" && (js.method == "success" || js.method == "failure")) {
      var ihid = js.idem;
      if (ihid) {
        console.log("have idem", ihid);
        this.bridge.unlock("ihid " + ihid);
      }
    }
  } catch (e) {
    console.log("error interpreting msg", e);
  }
};
ZinTest.prototype.sentMessage = function(msg) {
  console.log("connection sent message", msg);
  try {
    var js = JSON.parse(msg);
    if (js && js.args && js.args.length > 0) {
      var ihid = js.args[js.args.length - 1]._ihid;
      if (ihid) {
        console.log("have ihid", ihid);
        this.bridge.lock("ihid " + ihid);
      }
    }
  } catch (e) {
    console.log("error interpreting msg", e);
  }
};
ZinTest.prototype.ensureServer = function(zn, u) {
  var self = this;
  if (!this.servers[zn]) {
    var offset = Object.keys(this.servers).length;
    var baseport = 18080 + offset * 100;
    self.bridge.lock("base port");
    this.bridge.send({ action: "baseport", conn: this.conn, requestId: this.bridge.nextRequestId(() => {
      self.bridge.unlock("base port");
      self.beginConnect(zn, u, baseport);
    }), args: [zn, baseport] });
  } else {
    self.waitToConnect(zn);
  }
};
ZinTest.prototype.beginConnect = function(zn, u, baseport) {
  var self = this;
  this.servers[zn] = { domain: zn, baseport, user: u };
  this.servers[zn].conn = this.bridge.runner.broker.connectToServer(
    "ws://localhost:" + (baseport + 5) + "?token=tok_" + u + "." + zn + "&secret=secret_" + u,
    {
      status: (msg) => self.connStatus(msg),
      sentMessage: (msg) => self.sentMessage(msg),
      recvMessage: (msg) => self.recvMessage(msg)
    }
  );
  self.waitToConnect(zn);
};
ZinTest.prototype.waitToConnect = function(zn) {
  var self = this;
  this.bridge.lock("connecting to " + zn + " with base port " + this.servers[zn].baseport);
  var srv = this.servers[zn].conn;
  var max = 10;
  var unl = () => {
    console.log("checking if", srv.conn, "is connected yet");
    this.bridge.log("checking if connected to " + zn);
    if (srv.conn != null && srv.conn.readyState == 1) {
      self.bridge.unlock("connected ziwsh");
    } else if (--max > 0)
      setTimeout(unl, 100);
    else {
      console.log("never connected");
      this.bridge.error("never connected to " + zn);
    }
  };
  unl();
};
var installer = function(bridge) {
  bridge.connectModule("ziniki", (runner, connName) => {
    console.log("have module connection", connName);
    runner.bindModule("ziniki", new ZinTest(runner, bridge, connName));
  });
};
export {
  installer
};
